using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Net;
using CMS.Helpers;

namespace CMS.UIControls
{
    /// <summary>
    /// This class encapsulates reCAPTCHA UI and logic into an ASP.NET server control.
    /// </summary>
    [ToolboxData("<{0}:InvisibleReCaptchaControl runat=\"server\" />")]
    public class InvisibleReCaptchaControl : WebControl, IValidator, INamingContainer
    {
        #region "Private Fields"

        private const string RECAPTCHA_API_URL = "https://www.google.com/recaptcha/api.js?onload=onloadRecaptchaCallback";
        private const string RECAPTCHA_RESPONSE_FIELD = "g-recaptcha-response";

        private InvisibleReCaptchaResponse mRecaptchaResponse;
        private string mErrorMessage;

        #endregion


        #region "Public Properties"

        /// <summary>
        /// The public site key from https://www.google.com/recaptcha/admin.
        /// </summary>
        [Category("Settings")]
        [Description("The public site key from https://www.google.com/recaptcha/admin.")]
        public string SiteKey
        {
            get;
            set;
        }


        /// <summary>
        /// The private secret from https://www.google.com/recaptcha/admin.
        /// </summary>
        [Category("Settings")]
        [Description("The private secret from https://www.google.com/recaptcha/admin.")]
        public string Secret
        {
            get;
            set;
        }


        /// <summary>
        /// Optional. The type of CAPTCHA to serve. Possible values are 'audio' and 'image'. Default is 'image'.
        /// </summary>
        [Category("Settings")]
        [DefaultValue("")]
        [Description("Optional. The type of CAPTCHA to serve. Possible values are 'audio' and 'image'. Default is 'image'.")]
        public string DataType
        {
            get;
            set;
        }


        /// <summary>
        /// reCAPTCHA proxy used to validate.
        /// </summary>
        [Category("Settings")]
        [Description("Set this to override proxy used to validate reCAPTCHA.")]
        public IWebProxy Proxy
        {
            get;
            set;
        }

        /// <summary>
        /// Is clonned instance.
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public static bool IsMultipleInstance
        {
            get;
            private set;
        }
        
        #endregion


        #region Overriden Methods

        /// <summary>
        /// Control init event
        /// </summary>
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            if (!RequestHelper.IsPostBack())
            {
                SessionHelper.Remove("invisibleRecaptchaResponse");
            }
            Page.Validators.Add(this);
            IsMultipleInstance = false;
            ScriptManager.RegisterStartupScript(this, typeof(Page), UniqueID, "if (typeof(onloadRecaptchaCallback) === 'function') { onloadRecaptchaCallback(); }", true);
        }


        /// <summary>
        /// Control render event.
        /// </summary>
        /// <param name="writer">HtmlTextWriter parameter</param>
        protected override void Render(HtmlTextWriter writer)
        {
            // Render the div tag for nesting the reCaptcha
            writer.Write($"<div class='g-recaptcha' id='kenticoInvisibleRecaptcha' data-sitekey='{SiteKey}' data-callback='onloadRecaptchaCallback' data-size='invisible'> </div>");

            if (!IsMultipleInstance)
            {
                IsMultipleInstance = true;

                // Render the reCaptcha scripts
                
                var script = @"
                    var hasChallenged = false;
                    var formSubmitButtonId  = '';
                    window.onload = function () {
                            formSubmitButtonId = $('#kenticoInvisibleRecaptcha').closest(""div[id$='_pnlForm']"").find(""input[id$='_btnOK']"")[0].id;
                            $(""#"" + formSubmitButtonId).attr('onClick', 'return presentChallenge()');
                        }
                    function presentChallenge() {
                        if (!hasChallenged) {
                            grecaptcha.execute();
                            return false;
                        }
                    }
                                        
                    var onloadRecaptchaCallback = function (response) {
                            if(response !== undefined){
                                hasChallenged = true;
                                $(""#"" + formSubmitButtonId).click();
                            }
                        };
                ";

                
                writer.Write("<script type='text/javascript' defer='defer'>" + script);
                writer.Write("</script>");

                writer.Write("<script src=\"{0}\" async defer></script>", RECAPTCHA_API_URL);
            }
            base.Render(writer);
        }

        #endregion


        #region IValidator Members

        /// <summary>
        /// Validation error message.
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public string ErrorMessage
        {
            get
            {
                return (mRecaptchaResponse != null) ? mRecaptchaResponse.ErrorMessage : mErrorMessage;
            }

            set
            {
                mErrorMessage = value;
            }
        }


        /// <summary>
        /// Indicates if validation was successful.
        /// </summary>
        [Browsable(false)]
        [DesignerSerializationVisibilityAttribute(DesignerSerializationVisibility.Hidden)]
        public bool IsValid
        {
            get
            {
                return ((mRecaptchaResponse == null) || mRecaptchaResponse.IsValid);
            }

            set
            {
                throw new NotImplementedException("IsValid property is not settable. This property is populated automatically.");
            }
        }


        /// <summary>
        /// Perform validation of reCAPTCHA.
        /// </summary>
        public void Validate()
        {
            if (!string.IsNullOrEmpty(Secret) && !string.IsNullOrEmpty(SiteKey))
            {
                if (RequestHelper.IsPostBack() && Visible && Enabled && (Context != null))
                {
                    mRecaptchaResponse = SessionHelper.GetValue("invisibleRecaptchaResponse") as InvisibleReCaptchaResponse;
                    if (mRecaptchaResponse == null)
                    {
                        if (Visible && Enabled)
                        {
                            var validator = new InvisibleReCaptchaValidator
                            {
                                Secret = Secret,
                                RemoteIP = RequestContext.UserHostAddress,
                                Response = GetCoupledRequestFormValue(RECAPTCHA_RESPONSE_FIELD),
                                Proxy = Proxy
                            };

                            mRecaptchaResponse = validator.Validate();
                            SessionHelper.SetValue("invisibleRecaptchaResponse", mRecaptchaResponse);
                        }
                    }
                }
                else
                {
                    mRecaptchaResponse = InvisibleReCaptchaResponse.Valid;
                }
            }
            else
            {
                mRecaptchaResponse = InvisibleReCaptchaResponse.MissingSecret;
                ErrorMessage = "New reCAPTCHA needs to be configured with a secret & site key.";
            }

        }

        #endregion


        #region "Helper methods"

        /// <summary>
        /// Get form field value
        /// </summary>
        /// <param name="formField">Field name to get its value from</param>
        private string GetCoupledRequestFormValue(string formField)
        {
            // Get form value
            var fieldValue = Context.Request.Form[formField];
            if (!string.IsNullOrEmpty(fieldValue))
            {
                var fieldValuesArray = fieldValue.Split(new[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                if (fieldValuesArray.Length > 0)
                {
                    fieldValue = fieldValuesArray[0];
                }
                else
                {
                    fieldValue = string.Empty;
                }
            }

            return fieldValue;
        }

        #endregion
    }
}