using System;
using System.Net;
using System.Text;
using CMS.IO;
using Newtonsoft.Json.Linq;

namespace CMS.UIControls
{
    /// <summary>
    /// Encapsulates a response from new reCAPTCHA web service.
    /// </summary>
    public class InvisibleReCaptchaResponse
    {
        #region "Variables"

        /// <summary>
        /// Valid solution.
        /// </summary>
        public static readonly InvisibleReCaptchaResponse Valid = new InvisibleReCaptchaResponse(true, string.Empty);

        /// <summary>
        /// Invalid solution.
        /// </summary>
        public static readonly InvisibleReCaptchaResponse Invalid = new InvisibleReCaptchaResponse(false, string.Empty);

        /// <summary>
        /// Invalid secret.
        /// </summary>
        public static readonly InvisibleReCaptchaResponse InvalidSecret = new InvisibleReCaptchaResponse(false, "The secret parameter is invalid or malformed.");

        /// <summary>
        /// Invalid response.
        /// </summary>
        public static readonly InvisibleReCaptchaResponse InvalidResponse = new InvisibleReCaptchaResponse(false, "The response parameter is invalid or malformed.");

        /// <summary>
        /// Missing secret.
        /// </summary>
        public static readonly InvisibleReCaptchaResponse MissingSecret = new InvisibleReCaptchaResponse(false, "The secret parameter is missing.");

        /// <summary>
        /// Missing response.
        /// </summary>
        public static readonly InvisibleReCaptchaResponse MissingResponse = new InvisibleReCaptchaResponse(false, "The response parameter is missing.");

        #endregion


        #region "Properties"

        /// <summary>
        /// Indicates if response is valid.
        /// </summary>
        public bool IsValid
        {
            get;
            private set;
        }


        /// <summary>
        /// Gets response error message.
        /// </summary>
        public string ErrorMessage
        {
            get;
            private set;
        }

        #endregion


        #region "Methods"

        /// <summary>
        /// Initializes a new instance of the <see cref="RecaptchaResponse"/> class.
        /// </summary>
        /// <param name="isValid">Value indicates whether submitted reCAPTCHA is valid.</param>
        /// <param name="errorMessage">Error message returned from reCAPTCHA web service.</param>
        internal InvisibleReCaptchaResponse(bool isValid, string errorMessage)
        {
            InvisibleReCaptchaResponse templateResponse = null;

            if (IsValid)
            {
                templateResponse = Valid;
            }
            else
            {
                switch (errorMessage)
                {
                    case "missing-input-secret":
                        templateResponse = MissingSecret;
                        break;
                    case "invalid-input-secret":
                        templateResponse = InvalidSecret;
                        break;
                    case "missing-input-response":
                        templateResponse = MissingResponse;
                        break;
                    case "invalid-input-response":
                        templateResponse = InvalidResponse;
                        break;
                    case null:
                        throw new ArgumentNullException("errorMessage");
                }
            }

            // Check if template response is not empty
            if (templateResponse != null)
            {
                IsValid = templateResponse.IsValid;
                ErrorMessage = templateResponse.ErrorMessage;
            }
            else
            {
                IsValid = isValid;
                ErrorMessage = errorMessage;
            }
        }


        /// <summary>
        /// Creates InvisibleReCaptchaResponse object from JSON response.
        /// </summary>
        /// <param name="response">Response from google servers in JSON</param>
        public static InvisibleReCaptchaResponse ParseResponse(WebResponse response)
        {
            using (var stream = StreamReader.New(response.GetResponseStream(), Encoding.UTF8))
            {
                var joResponse = JObject.Parse(stream.ReadToEnd());

                var isValid = false;
                var success = joResponse["success"];
                if (success != null)
                {
                    isValid = success.Value<bool>();
                }

                var errorMessage = "";
                var errorCodes = (JArray)joResponse["error-codes"];
                if (errorCodes != null)
                {
                    errorMessage = errorCodes[0].Value<string>();
                }

                return new InvisibleReCaptchaResponse(isValid, errorMessage);
            }
        }


        /// <summary>
        /// Check if two objects are equal.
        /// </summary>
        /// <param name="obj">Object to check</param>
        public override bool Equals(object obj)
        {
            var other = (InvisibleReCaptchaResponse)obj;
            if (other == null)
            {
                return false;
            }

            return ((other.IsValid == IsValid) && (other.ErrorMessage == ErrorMessage));
        }


        /// <summary>
        /// Returns object hash code.
        /// </summary>
        public override int GetHashCode()
        {
            return IsValid.GetHashCode() ^ ErrorMessage.GetHashCode();
        }

        #endregion
    }
}